#ifndef KATOPTRON_DOMAIN_H
#define KATOPTRON_DOMAIN_H

#include "katoptron.h"
#include "Map.h"
#include "ElementsList.h"
#include "NodesList.h"
#include "MaterialsList.h"
#include "ContactsList.h"
#include "wRandomField.h"

namespace katoptron
{

/**
 * @brief Class which is used to store all the information related to the discretized domain:
 * <ul>
 *  <li> the list of the local elements,
 *  <li> the list of the nodes,
 *  <li> the list of the materials,
 *  <li> the list of the contact interfaces,
 *  <li> the list of the random fields.
 * </ul>
 */
template <typename scalar>
class Domain
{
public:
    Teuchos::RCP<ElementsList> elementsList;
    Teuchos::RCP<NodesList> nodesList;
    Teuchos::RCP<MaterialsList<scalar>> materialsList;
    Teuchos::RCP<ContactsList> contactsList;
    Teuchos::RCP<RandomField<scalar, Kokkos::DefaultExecutionSpace>> random_field;

    Domain(Problem &pbl,
           Teuchos::RCP<Map> map,
           Teuchos::RCP<Teuchos::ParameterList> randomParams,
           Kokkos::View<scalar *, Kokkos::LayoutLeft> m_rv_values);

    ~Domain()
    {
        std::cout << "~Domain()\n";
    }
};

/**
 * @brief Domain constructor
 * 
 * This constructor initializes the lists of elements, nodes, materials, contact interfaces, and random fields.
 * 
 * Arguments:
 * <ul>
 *  <li> A problem object,
 *  <li> An RCP to a Map object,
 *  <li> An RCP to a ParameterList object which describes the random fields,
 *  <li> A Kokkos View of the values used in the discretization of the random fields.
 * </ul>
 */
template <typename scalar>
Domain<scalar>::Domain(Problem &pbl,
                       Teuchos::RCP<Map> map,
                       Teuchos::RCP<Teuchos::ParameterList> randomParams,
                       Kokkos::View<scalar *, Kokkos::LayoutLeft> m_rv_values)
{
    elementsList = Teuchos::rcp(new ElementsList(pbl, map));
    nodesList = Teuchos::rcp(new NodesList(pbl, map));
    materialsList = Teuchos::rcp(new MaterialsList<scalar>(pbl, map));
    contactsList = Teuchos::rcp(new ContactsList(pbl, map, elementsList, nodesList));
    random_field = Teuchos::rcp(new RandomField<scalar, Kokkos::DefaultExecutionSpace>(randomParams, m_rv_values));
}

}; // namespace katoptron

#endif //KATOPTRON_DOMAIN_H
